@@ -186,27 +186,15 @@ class HashContext(object):
 @six.add_metaclass(abc.ABCMeta)
 class RSAPrivateKey(object):
     @abc.abstractmethod
-    def signer(self, padding, algorithm, backend):
+    def signer(self, padding, algorithm):
         """
         Returns an AsymmetricSignatureContext used for signing data.
         """
 
-    @abc.abstractproperty
-    def modulus(self):
+    @abc.abstractmethod
+    def decrypt(self, ciphertext, padding):
         """
-        The public modulus of the RSA key.
-        """
-
-    @abc.abstractproperty
-    def public_exponent(self):
-        """
-        The public exponent of the RSA key.
-        """
-
-    @abc.abstractproperty
-    def private_exponent(self):
-        """
-        The private exponent of the RSA key.
+        Decrypts the provided ciphertext.
         """
 
     @abc.abstractproperty
@@ -221,77 +209,28 @@ class RSAPrivateKey(object):
         The RSAPublicKey associated with this private key.
         """
 
-    @abc.abstractproperty
-    def n(self):
-        """
-        The public modulus of the RSA key. Alias for modulus.
-        """
 
-    @abc.abstractproperty
-    def p(self):
+@six.add_metaclass(abc.ABCMeta)
+class RSAPrivateKeyWithNumbers(RSAPrivateKey):
+    @abc.abstractmethod
+    def private_numbers(self):
         """
-        One of the two primes used to generate d.
-        """
-
-    @abc.abstractproperty
-    def q(self):
-        """
-        One of the two primes used to generate d.
-        """
-
-    @abc.abstractproperty
-    def d(self):
-        """
-        The private exponent. This can be calculated using p and q. Alias for
-        private_exponent.
-        """
-
-    @abc.abstractproperty
-    def dmp1(self):
-        """
-        A Chinese remainder theorem coefficient used to speed up RSA
-        calculations.  Calculated as: d mod (p-1)
-        """
-
-    @abc.abstractproperty
-    def dmq1(self):
-        """
-        A Chinese remainder theorem coefficient used to speed up RSA
-        calculations.  Calculated as: d mod (q-1)
-        """
-
-    @abc.abstractproperty
-    def iqmp(self):
-        """
-        A Chinese remainder theorem coefficient used to speed up RSA
-        calculations. The modular inverse of q modulo p
-        """
-
-    @abc.abstractproperty
-    def e(self):
-        """
-        The public exponent of the RSA key. Alias for public_exponent.
+        Returns an RSAPrivateNumbers.
         """
 
 
 @six.add_metaclass(abc.ABCMeta)
 class RSAPublicKey(object):
     @abc.abstractmethod
-    def verifier(self, signature, padding, algorithm, backend):
+    def verifier(self, signature, padding, algorithm):
         """
         Returns an AsymmetricVerificationContext used for verifying signatures.
         """
 
-    @abc.abstractproperty
-    def modulus(self):
+    @abc.abstractmethod
+    def encrypt(self, plaintext, padding):
         """
-        The public modulus of the RSA key.
-        """
-
-    @abc.abstractproperty
-    def public_exponent(self):
-        """
-        The public exponent of the RSA key.
+        Encrypts the given plaintext.
         """
 
     @abc.abstractproperty
@@ -300,63 +239,31 @@ class RSAPublicKey(object):
         The bit length of the public modulus.
         """
 
-    @abc.abstractproperty
-    def n(self):
-        """
-        The public modulus of the RSA key. Alias for modulus.
-        """
 
-    @abc.abstractproperty
-    def e(self):
+@six.add_metaclass(abc.ABCMeta)
+class RSAPublicKeyWithNumbers(RSAPublicKey):
+    @abc.abstractmethod
+    def public_numbers(self):
         """
-        The public exponent of the RSA key. Alias for public_exponent.
+        Returns an RSAPublicNumbers
         """
 
 
 @six.add_metaclass(abc.ABCMeta)
 class DSAParameters(object):
-    @abc.abstractproperty
-    def modulus(self):
+    @abc.abstractmethod
+    def generate_private_key(self):
         """
-        The prime modulus that's used in generating the DSA keypair and used
-        in the DSA signing and verification processes.
+        Generates and returns a DSAPrivateKey.
         """
 
-    @abc.abstractproperty
-    def subgroup_order(self):
-        """
-        The subgroup order that's used in generating the DSA keypair
-        by the generator and used in the DSA signing and verification
-        processes.
-        """
 
-    @abc.abstractproperty
-    def generator(self):
+@six.add_metaclass(abc.ABCMeta)
+class DSAParametersWithNumbers(DSAParameters):
+    @abc.abstractmethod
+    def parameter_numbers(self):
         """
-        The generator that is used in generating the DSA keypair and used
-        in the DSA signing and verification processes.
-        """
-
-    @abc.abstractproperty
-    def p(self):
-        """
-        The prime modulus that's used in generating the DSA keypair and used
-        in the DSA signing and verification processes. Alias for modulus.
-        """
-
-    @abc.abstractproperty
-    def q(self):
-        """
-        The subgroup order that's used in generating the DSA keypair
-        by the generator and used in the DSA signing and verification
-        processes. Alias for subgroup_order.
-        """
-
-    @abc.abstractproperty
-    def g(self):
-        """
-        The generator that is used in generating the DSA keypair and used
-        in the DSA signing and verification processes. Alias for generator.
+        Returns a DSAParameterNumbers.
         """
 
 
@@ -374,18 +281,6 @@ class DSAPrivateKey(object):
         The DSAPublicKey associated with this private key.
         """
 
-    @abc.abstractproperty
-    def x(self):
-        """
-        The private key "x" in the DSA structure.
-        """
-
-    @abc.abstractproperty
-    def y(self):
-        """
-        The public key.
-        """
-
     @abc.abstractmethod
     def parameters(self):
         """
@@ -393,6 +288,15 @@ class DSAPrivateKey(object):
         """
 
 
+@six.add_metaclass(abc.ABCMeta)
+class DSAPrivateKeyWithNumbers(DSAPrivateKey):
+    @abc.abstractmethod
+    def private_numbers(self):
+        """
+        Returns a DSAPrivateNumbers.
+        """
+
+
 @six.add_metaclass(abc.ABCMeta)
 class DSAPublicKey(object):
     @abc.abstractproperty
@@ -401,12 +305,6 @@ class DSAPublicKey(object):
         The bit length of the prime modulus.
         """
 
-    @abc.abstractproperty
-    def y(self):
-        """
-        The public key.
-        """
-
     @abc.abstractmethod
     def parameters(self):
         """
@@ -414,6 +312,15 @@ class DSAPublicKey(object):
         """
 
 
+@six.add_metaclass(abc.ABCMeta)
+class DSAPublicKeyWithNumbers(DSAPublicKey):
+    @abc.abstractmethod
+    def public_numbers(self):
+        """
+        Returns a DSAPublicNumbers.
+        """
+
+
 @six.add_metaclass(abc.ABCMeta)
 class AsymmetricSignatureContext(object):
     @abc.abstractmethod
@@ -489,3 +396,63 @@ class CMACContext(object):
         """
         Return a CMACContext that is a copy of the current context.
         """
+
+
+@six.add_metaclass(abc.ABCMeta)
+class EllipticCurve(object):
+    @abc.abstractproperty
+    def name(self):
+        """
+        The name of the curve. e.g. secp256r1.
+        """
+
+    @abc.abstractproperty
+    def key_size(self):
+        """
+        The bit length of the base point of the curve.
+        """
+
+
+@six.add_metaclass(abc.ABCMeta)
+class EllipticCurveSignatureAlgorithm(object):
+    @abc.abstractproperty
+    def algorithm(self):
+        """
+        The digest algorithm used with this signature.
+        """
+
+
+@six.add_metaclass(abc.ABCMeta)
+class EllipticCurvePrivateKey(object):
+    @abc.abstractmethod
+    def signer(self, signature_algorithm):
+        """
+        Returns an AsymmetricSignatureContext used for signing data.
+        """
+
+    @abc.abstractmethod
+    def public_key(self):
+        """
+        The EllipticCurvePublicKey for this private key.
+        """
+
+    @abc.abstractproperty
+    def curve(self):
+        """
+        The EllipticCurve that this key is on.
+        """
+
+
+@six.add_metaclass(abc.ABCMeta)
+class EllipticCurvePublicKey(object):
+    @abc.abstractmethod
+    def verifier(self, signature, signature_algorithm):
+        """
+        Returns an AsymmetricVerificationContext used for signing data.
+        """
+
+    @abc.abstractproperty
+    def curve(self):
+        """
+        The EllipticCurve that this key is on.
+        """
